#!/bin/bash
VER="2.0"
################################################################
# KTMF by Turranius ( http://www.grandis.nu/glftpd/ )          #
# Please read the README for documentation.                    #
# It should be in the same package.                            #
# Note: If you are upgrading from 1.1.1 or earlier, you need   #
#       the new tur-ftpwho.                                    #
################################################################
# Changelog:                                                   #
# 2.0   : Add: Added the dual upload module. See README.       #
#         Add: Added LOG. Some people apparently wanted it so  #
#              here it is. It will log all kicks and whatever  #
#              it does to its own logfile. Disable totally by  #
#              setting LOG=""                                  #
#         Add: Added a verification check before kicking or    #
#              announcing. This should eliminate any misfires  #
#              you might have seen in the past.                #
#         Add: Some more initial info when running DEBUG mode. #
#         Add: If you run this with 'ktmf.sh test', it will    #
#              set DEBUG, TEST and SHOWEXCLUDES to TRUE, no    #
#              matter what they are set to in the script. Good #
#              if you want to test it from shell, but are too  #
#              lazy to change them to TRUE in the script.      #
#         Fix: Sometimes it counted the number of users wrong, #
#              resulting in kicks or whatever when there were  #
#              less users on site then specified.              #
#              It could also cause weird results, such as      #
#              trying to kick parts of a filename or command.  #
# 1.2   : Fix: Exclude error again. Should be the last one.    #
#         Add: Ability to exclude users who are in pre dirs.   #
#              This requires the new tur-ftpwho to work!       #
# 1.1.1 : Fix: Minor error. If debug was FALSE, it waited for  #
#              DELAYDUALDL even if it did not find anyone dual #
#              downloading.                                    #
#         Add: Added possibility to set KILLGHOST="" to        #
#              disable the function totally.                   #
#         Add: If DEBUG and SHOWEXCLUDES are TRUE, it will     #
#              make a nicer output of excluded users.          #
# 1.1   : Add: LIMITTODUALKICK. You can now set a usercount    #
#              that must be on the site before it starts to    #
#              kick for dual downloading, like for the others. #
#         Add: Added BANDUAL to Dual Downloading so you can    #
#              give users a tempban there too. Of course,      #
#              DUALBANTIME comes with that one too.            #
#         Fix: If Temp banning is enabled on any of the        #
#              functions, it will find ALL pids of a user and  #
#              kick them all. Why?:                            #
#              Say that hes slowdownloading and are uploading  #
#              in another thread.                              #
#              Before, if he was tempbanned, he could still    #
#              upload on the other thread. Then when he was    #
#              unbanned, everything he did in that time would  #
#              be lost. Now the uploading thread will be       #
#              kicked too, but only if tempbanning is enabled. #
#         Fix: Added some more checks to make sure it will run #
#         Fix: Fixed spelling errors in outputs and in README. #
#                                                              #
# 1.0.5 : Fix: If /glftpd/bin was not in the path, it did not  #
#         find tur-ftpwho. Fixed by adding WHOBIN.             #
# 1.0.4 : Fix: Even more exclude merge problems fixed. hehe.   #
# 1.0.3 : Fix: Display bug if FOLDEREX was "".                 #
#         Fix: More exclude merging problems fixes.            #
# 1.0.2 : Fix: Bug with userexcludes if GEXCLUDEON was FALSE.  #
#         Does not cause locking if it cant find any folder.   #
# 1.0.1 : Just added FOLDEREX.                                 #
# 1.0   : Initial release. What you see is what you get.       #
################################################################
# Settings: (Check the README for more detailed instructions.) #

WHOBIN=/glftpd/bin/tur-ftpwho          # Full path to tur-ftpwho.
GLLOG=/glftpd/ftp-data/logs/glftpd.log # Logfile to write to for bot announces.
LOG=/glftpd/ftp-data/logs/ktmf.log     # Kick/whats done log for ktmf.
USERFOLDER=/glftpd/ftp-data/users      # Path to user files.
TEMPPATH=/glftpd/tmp                   # Temporary folder to store stuff in.
BYEFOLDER=/glftpd/ftp-data/byefiles    # Path to byefiles folder incase someone gets banned :)
KILLGHOST="/glftpd/bin/killghost -r /etc/glftpd.conf" # See README
LOCKFILE=$TEMPPATH/KTMF.lock           # Lockfile, so it does not run twice.
FOLDEREX="disabled|backup"             # If you have folders in your userdir, add them here.
                                       # Separated with a '|' (without '').

PREFOLDERS="
/site/incoming/GROUPS 
/site/incoming/VCD/GROUPS 
/site/incoming/XXX/GROUPS
/site/incoming/DIVX/GROUPS
/site/incoming/SVCD/GROUPS
/site/incoming/ISO-UTILS/GROUPS
/site/incoming/0DAYS/GROUPS
"

EXCLUDE="turranius|per"  # Users to exclude from all parts. | separated.
GEXCLUDEON="TRUE"        # Check for group excludes? TRUE/FALSE
GEXCLUDE="SiTEOPS OMEGA" # Groups to exclude from all parts. Space separated!

## Dual Downloading section ##################################################################

DUALDL="TRUE"            # Dual Download kicker enabled? TRUE/FALSE
DELAYDUALDL="45"         # Time in seconds to wait before verifying dual downloading.
LIMITTODUALKICK="1"      # Minumum number of users on site before kicking anyone (1=always).
CREDITLOSSDD="FALSE"     # Loose credits for dual downloading? TRUE/FALSE
CREDITLOSSDDMB="1000"    # Credits in MB to loose if you dual download (if above is TRUE).
TAKE1LOGIN="FALSE"       # Loose one login? TRUE/FALSE
TAKE1DL="FALSE"          # Set -1 max_sim_down? TRUE/FALSE
BANDUAL="FALSE"          # Dual downloaders get tempbanned? TRUE/FALSE
DUALBANTIME="60"         # If above is true, how many seconds to ban then?
BOTDUALDL="TRUE"         # Announce kick in logfile? TRUE/FALSE
BOTDUALDLW="TRUE"        # Announce early warning on kick in logfile? TRUE/FALSE
DUALEXCLUDE=""           # Users to specifically exclude from dual DL kick, | separated.
DUALGEXCLUDE=""          # Groups to exclude from dual DL only. Space separated!

## Dual Upload section #######################################################################

DUALUP="TRUE"            # Dual Upload kicker enabled? TRUE/FALSE
DELAYDUALUP="45"         # Time in seconds to wait before verifying dual uploading.
LIMITTODUALUPKICK="1"    # Minumum number of users on site before kicking anyone (1=always).
CREDITLOSSDU="FALSE"     # Loose credits for dual downloading? TRUE/FALSE
CREDITLOSSDUMB="1000"    # Credits in MB to loose if you dual upload (if above is TRUE).
TAKE1LOGINDU="FALSE"     # Loose one login? TRUE/FALSE
TAKE1UP="FALSE"          # Set -1 max_sim_up? TRUE/FALSE
BANDUALUP="FALSE"        # Dual uploaders get tempbanned? TRUE/FALSE
DUALUPBANTIME="60"       # If above is true, how many seconds to ban then?
BOTDUALUP="TRUE"         # Announce kick in logfile? TRUE/FALSE
BOTDUALUPW="TRUE"        # Announce early warning on kick in logfile? TRUE/FALSE
DUALUPEXCLUDE=""         # Users to specifically exclude from dual UP kick, | separated.
DUALUPGEXCLUDE=""        # Groups to exclude from dual UP only. Space separated!

## Idle Kill section #########################################################################

IDLEKILL="TRUE"          # Idle kicker enabled? TRUE/FALSE
DELAYIDLEKILL="30"       # Time in seconds to wait before verifying idle users.
LIMITTOIDLEKICK="20"     # Minumum number of user on site before starting kicking people.
CREDITLOSSIK="FALSE"     # Loose credits for idling after above times? TRUE/FALSE
TIMESBEFOREBANI="1"      # How many concurrent times to wait before taking creds?
CREDITLOSSIKMB="1000"    # Credits in MB to loose if you idle. (if above is TRUE).
BANIDLE="FALSE"          # Ban users for a while? TRUE/FALSE
TIMESTOBAN="1"           # Number of times the user must have been idle before banning.
IDLEBANTIME="30"         # Seconds to wait before relasing ban.
BOTIDLE="TRUE"           # Announce kick in logfile? TRUE/FALSE
BOTIDLEW="FALSE"         # Announce early warning on kick in logfile? TRUE/FALSE
IDLEEXCLUDE=""           # Users to specifically exclude from IDLE kick, | separated.
IDLEGEXCLUDE=""          # Groups to exclude from dual DL only. Space separated!

## Slow Download section #####################################################################

SLOWDLKICK="TRUE"        # Slow download kicker enabled? TRUE/FALSE
DELAYSLOWDL="20"         # Time in seconds to wait before verifying slow downloaders.
SPEEDLIMIT="25"          # Whats the minimum speed before kicking in kb/sec?
LIMITTOKICK="20"         # Minumum number of user on site before starting kicking people.
BAN="FALSE"              # Ban the users for a specified time for slow downloading? TRUE/FALSE
BANTIME="60"             # Time in seconds before releasing ban on slow downloaders.
BOTSLOW="TRUE"           # Announce kick in logfile? TRUE/FALSE
BOTSLOWW="FALSE"         # Announce early warning on kick in logfile? TRUE/FALSE
SLOWEXCLUDE=""           # Users to specifically exclude from slow dl kick, | separated.
SLOWGEXCLUDE=""          # Groups to exclude from Slow dl kick only. Space separated!

## Generic section ###########################################################################

BOTTRIGGER="KTMF:"       # Trigger for bot announces. Your bot announce script trigger.
DEBUG="TRUE"             # Will show whats happening in shell. TRUE/FALSE
TEST="TRUE"              # If TRUE, dont actually kick anyone. Just pretend to. TRUE/FALSE
SHOWEXCLUDES="TRUE"      # If DEBUG is TRUE, this will also show who it will exclude.


####################################################################################
# No changes below here should be needed, but you are free to play around ofcourse #
####################################################################################

if [ "$1" = "test" ]; then
  DEBUG="TRUE"
  TEST="TRUE"
  SHOWEXCLUDES="TRUE"
fi

####################################################################################
# General check if paths and files are correct.                                    #
####################################################################################

if [ "$DEBUG" = "TRUE" ]; then
  msg="DEBUG is ON."
  if [ "$TEST" = "TRUE" ]; then
    msg="$msg TEST is ON."
  else
    msg="$msg TEST is OFF."
  fi
  if [ "$SHOWEXCLUDES" = "TRUE" ]; then
    msg="$msg SHOWEXCLUDES is ON."
  else
    msg="$msg SHOWEXCLUDES is OFF."
  fi
  echo "--[ Welcome to KTMF $VER. $msg ]--"
  if [ "$DUALDL" = "TRUE" ]; then
    msg="DualDownload is ON."
  else
    msg="DualDownload is OFF."
  fi
  if [ "$DUALUP" = "TRUE" ]; then
    msg="$msg DualUpload is ON."
  else
    msg="$msg DualUpload is OFF."
  fi
  if [ "$IDLEKILL" = "TRUE" ]; then
    msg="$msg IdleKill is ON."
  else
    msg="$msg Idlekill is OFF."
  fi
  if [ "$SLOWDLKICK" = "TRUE" ]; then
    msg="$msg SlowKick is ON."
  else
    msg="$msg Slowkick is OFF."
  fi
  echo "--[ ModuleInfo: $msg ]--"
  echo ""
fi

touch $GLLOG
if [ -e $GLLOG ]; then
  OK="YEPP"
else
  echo "Cant find log file $GLLOG. Check settings."
  exit 0
fi
  
if [ -e $WHOBIN ]; then
  OK=YEPP
else
  echo "Cant find $WHOBIN. Make sure its there and executable."
  exit 0
fi

touch $TEMPPATH/testtouch.tmp
if [ -e "$TEMPPATH/testtouch.tmp" ]; then
  rm -f $TEMPPATH/testtouch.tmp
else
  echo "Cant create files in $TEMPPATH. Check permissions."
  exit 0
fi

if [ -e "$BYEFOLDER" ]; then
  OK=YEPP
else
  echo "Cant find $BYEFOLDER for byefiles. Check settings."
  exit 0
fi

touch $BYEFOLDER/testtouch.tmp
if [ -e "$BYEFOLDER/testtouch.tmp" ]; then
  rm -f $BYEFOLDER/testtouch.tmp
else
  echo "Cant create files in $BYEFOLDER. Check permissions."
  exit 0
fi

touch $USERFOLDER/testtouch.tmp
if [ -e "$USERFOLDER/testtouch.tmp" ]; then
  rm -f $USERFOLDER/testtouch.tmp
else
  echo "Cant create files in $USERFOLDER. Check permissions."
  exit 0
fi

if [ "$LOG" != "" ]; then
  touch $LOG
  if [ -e "$LOG" ]; then
    OK="YEPP"
  else
    echo "Set to log to $LOG, but I cant find, or dont have access, to that file."
  fi
fi

if [ "$LOCKFILE" != "" ]; then
  if [ -e $LOCKFILE ]; then
    echo "Script already running? Delete $LOCKFILE if you are sure its not."
    exit 0
  fi

  touch $LOCKFILE
  if [ -e $LOCKFILE ]; then
    OK="YEPP"
  else
    echo "Cant create lockfile $LOCKFILE. Check settings and permissions."
    exit 0
  fi
fi

#####################################################################################
## Excluding from groups, if enabled,                                               #
#####################################################################################

if [ "$EXCLUDE" != "" ]; then
  if [ "$DUALEXCLUDE" != "" ]; then
    EXCLUDE1="$EXCLUDE|$DUALEXCLUDE"
  else
    EXCLUDE1="$EXCLUDE"
  fi
else
  if [ "$DUALEXCLUDE" != "" ]; then
    EXCLUDE1="$DUALEXCLUDE"
  else
    EXCLUDE1="$EXCLUDE"
  fi
fi

if [ "$EXCLUDE" != "" ]; then
  if [ "$IDLEEXCLUDE" != "" ]; then
    EXCLUDE2="$EXCLUDE|$IDLEEXCLUDE"
  else
    EXCLUDE2="$EXCLUDE"
  fi
else
  if [ "$IDLEEXCLUDE" != "" ]; then
    EXCLUDE2="$IDLEEXCLUDE"
  else
    EXCLUDE2="$EXCLUDE"
  fi
fi

if [ "$EXCLUDE" != "" ]; then
  if [ "$SLOWEXCLUDE" != "" ]; then
    EXCLUDE3="$EXCLUDE|$SLOWEXCLUDE"
  else
    EXCLUDE3="$EXCLUDE"
  fi
else
  if [ "$SLOWEXCLUDE" != "" ]; then
    EXCLUDE3="$SLOWEXCLUDE"
  else
    EXCLUDE3="$EXCLUDE"
  fi
fi

if [ "$EXCLUDE" != "" ]; then
  if [ "$DUALUPEXCLUDE" != "" ]; then
    EXCLUDE4="$EXCLUDE|$DUALUPEXCLUDE"
  else
    EXCLUDE4="$EXCLUDE"
  fi
else
  if [ "$DUALUPEXCLUDE" != "" ]; then
    EXCLUDE4="$DUALUPEXCLUDE"
  else
    EXCLUDE4="$EXCLUDE"
  fi
fi

if [ "$GEXCLUDEON" = "TRUE" ]; then
  if [ "$DEBUG" = "TRUE" ]; then
    echo "Checking who is in what group, so we can exclude them. Hang on."
  fi

  LASTIDLEMATCH=""
  LASTDUALMATCH=""
  LASTSLOWMATCH=""
  LASTDUALUPWMATCH=""
  cd $USERFOLDER
  DUALGEXCLUDE="$GEXCLUDE $DUALGEXCLUDE"
  IDLEGEXCLUDE="$GEXCLUDE $IDLEGEXCLUDE"
  SLOWGEXCLUDE="$GEXCLUDE $SLOWGEXCLUDE"
  DUALUPGEXCLUDE="$GEXCLUDE $DUALUPGEXCLUDE"

  ## Making a lame fix.
  if [ "$FOLDEREX" = "" ]; then
    FOLDEREX="qQw98654321"
  fi

  for i in `ls -f -A | egrep -v $FOLDEREX`; do
    GROUP=`cat $i | grep -w GROUP | awk -F" " '{print $2}'`
    for e in $GROUP; do
      if [ "$DUALDL" = "TRUE" ]; then
        MATCH1="$( echo $DUALGEXCLUDE | grep -w $e )"
        if [ "$MATCH1" != "" ]; then
          if [ "$LASTDUALEMATCH" != "$i" ]; then
            if [ "$EXCLUDE1" = "" ]; then
              EXCLUDE1="$i"
              LASTDUALMATCH="$i"
            else
              EXCLUDE1="$EXCLUDE1|$i"
              LASTDUALMATCH="$i"
            fi
          fi
        fi
      fi
      if [ "$IDLEKILL" = "TRUE" ]; then
        MATCH2="$( echo $IDLEGEXCLUDE | grep -w $e )"
        if [ "$MATCH2" != "" ]; then
          if [ "$LASTIDLEMATCH" != "$i" ]; then
            if [ "$EXCLUDE2" = "" ]; then
              EXCLUDE2="$i"
              LASTIDLEMATCH="$i"
            else
              EXCLUDE2="$EXCLUDE2|$i"
              LASTIDLEMATCH="$i"
            fi
          fi
        fi
      fi
      if [ "$SLOWDLKICK" = "TRUE" ]; then
        MATCH3="$( echo $SLOWGEXCLUDE | grep -w $e )"
        if [ "$MATCH3" != "" ]; then
          if [ "$LASTSLOWMATCH" != "$i" ]; then
            if [ "$EXCLUDE3" = "" ]; then
              EXCLUDE3="$i"
              LASTSLOWMATCH="$i"
            else
              EXCLUDE3="$EXCLUDE3|$i"
              LASTSLOWMATCH="$i"
            fi
          fi
        fi
      fi
      if [ "$DUALUP" = "TRUE" ]; then
        MATCH4="$( echo $DUALUPGEXCLUDE | grep -w $e )"
        if [ "$MATCH4" != "" ]; then
          if [ "$LASTDUALUPMATCH" != "$i" ]; then
            if [ "$EXCLUDE4" = "" ]; then
              EXCLUDE4="$i"
              LASTDUALUPMATCH="$i"
            else
              EXCLUDE4="$EXCLUDE4|$i"
              LASTDUALUPMATCH="$i"
            fi
          fi
        fi
      fi
    done
  done
fi

if [ "$EXCLUDE1" = "" ]; then
  EXCLUDE1="NoBodyAtAll"
fi
if [ "$EXCLUDE2" = "" ]; then
  EXCLUDE2="NoBodyAtAll"
fi

if [ "$EXCLUDE3" = "" ]; then
  EXCLUDE3="NoBodyAtAll"
fi

if [ "$EXCLUDE4" = "" ]; then
  EXCLUDE4="NoBodyAtAll"
fi

#####################################################################################
## Dual Downloaders part.                                                           #
#####################################################################################

if [ "$DUALDL" = "TRUE" ]; then
  if [ "$DEBUG" = "TRUE" ]; then
    echo "-----------------------------------"
    echo "Start Dual DL kick part.           "
  fi

  LAST=""
  DUALS=""
  DUALS2=""
  
  if [ "$KILLGHOST" != "" ]; then
    $KILLGHOST
  fi

  USERSON="0"
  for online in `$WHOBIN | tr -d ' '`; do
    USERSON="$( expr "$USERSON" \+ "1" )"
  done

  if [ "$USERSON" -lt "$LIMITTODUALKICK" ]; then
    if [ "$DEBUG" = "TRUE" ]; then
      echo "Only $USERSON users online. Not running unless theres $LIMITTODUALKICK on."
    fi
    DUALGO="NO"
  else
    DUALGO="YES"
  fi

  if [ "$DEBUG" = "TRUE" ]; then
    if [ "$SHOWEXCLUDES" = "TRUE" ]; then
      if [ "$EXCLUDE1" != "" ]; then
        EXCLUDE1VIEW="$( echo $EXCLUDE1 | tr -s '|' ' ' )"
        echo " "
        echo "Based on excludes from groups and users, the following users wont be checked:"
        echo "$EXCLUDE1VIEW"
        echo " "
      fi
    fi
  fi

  if [ "$DUALGO" = "YES" ]; then
    for u in `$WHOBIN | tr -d ' '| grep -w Dn: | egrep -v $EXCLUDE1 | sort ` ;do
      USERNOW="$( echo $u | awk -F"^" '{print $1}')"
      if [ "$PREFOLDERS" != "" ]; then
        for prefolder in $PREFOLDERS; do
          VERPREFOLDER="$( echo $u | grep -w $prefolder )"
          if [ "$VERPREFOLDER" != "" ]; then
            if [ "$DEBUG" = "TRUE" ]; then
              echo "$USERNOW is in $prefolder and wont be checked"
            fi
            INPREFOLDERDL="YES"
          fi
        done
        VERPREFOLDER=""
        prefolder=""
      fi
      if [ "$INPREFOLDERDL" != "YES" ]; then
        if [ "$USERNOW" = "$LAST" ]; then
          VERIFY="$( echo $DUALS | grep $USERNOW )"
          if [ "$VERIFY" = "" ]; then
            if [ "$DEBUG" = "TRUE" ]; then
              echo "$USERNOW is dual downloading"
            fi
            if [ "$BOTDUALDLW" = "TRUE" ]; then
              if [ -e "$USERFOLDER/$USERNOW" ]; then
                echo "$(date +'%a %b %d %T %Y') $BOTTRIGGER \002-(DUALDL)- $USERNOW\002, early warning for dual download. Checking again in $DELAYDUALDL seconds." >> $GLLOG
              fi
              if [ "$LOG" != "" ]; then
                echo "$(date +%x' '%T): DUALDL: $USERNOW - Early warning for Dual downloading." >> $LOG
              fi
            fi
            if [ -e "$USERFOLDER/$USERNOW" ]; then
              DUALS="$USERNOW $DUALS"
            fi
          fi
        fi
        INPREFOLDERDL=""
      fi
      LAST=$USERNOW
    done

    if [ "$DUALS" = "" ]; then
      if [ "$DEBUG" = "TRUE" ]; then
        echo "No users dualling. Quitting dual DL part"
      fi
      NODUAL="TRUE"
    else
      if [ "$DEBUG" = "TRUE" ]; then
        echo "Going to check $DUALS after $DELAYDUALDL seconds."
      fi
    fi
  
    PIDS=""
    VERIFY=""

    if [ "$NODUAL" != "TRUE" ]; then
      sleep $DELAYDUALDL
      LAST=""
      USERNOW=""

      if [ "$DEBUG" = "TRUE" ]; then
        echo "-------------------------"
      fi

      for user in `echo $DUALS`; do
        for u in `$WHOBIN | tr -d ' ' | grep -w Dn: | grep -w $user | egrep -v $EXCLUDE1 | sort ` ;do
          USERNOW="$( echo $u | awk -F"^" '{print $1}')"
          USERPID="$( echo $u | awk -F"^" '{print $2}')"
          if [ -e "$USERFOLDER/$USERNOW" ]; then
            ok=yes
          else
            USERNOW="$RANDOM"
          fi
          if [ "$USERNOW" = "$LAST" ]; then
            VERIFY="$( echo $DUALS2 | grep $USERNOW )"
            if [ "$VERIFY" = "" ]; then
              if [ "$BOTDUALDL" = "TRUE" ]; then
                echo "$(date +'%a %b %d %T %Y') $BOTTRIGGER \002-(DUALDL)- $USERNOW\002 kicked for dual downloading!" >> $GLLOG
              fi
              if [ "$LOG" != "" ]; then
                echo "$(date +%x' '%T): DUALDL: $USERNOW kicked." >> $LOG
              fi
              if [ "$DEBUG" = "TRUE" ]; then
                echo "$USERNOW is still dual downloading"
              fi
              if [ "$CREDITLOSSDD" = "TRUE" ]; then
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "Taking $CREDITLOSSDDMB MB from $USERNOW"
                fi
                if [ "$LOG" != "" ]; then
                  echo "$(date +%x' '%T): DUALDL: $USERNOW lost $CREDITLOSSDDMB MB creds." >> $LOG
                fi
                if [ "$TEST" != "TRUE" ]; then
                  CREDITLOSSKB="$( expr "$CREDITLOSSDDMB" \* "1024" )"
                  BEFORECREDS="$(cat $USERFOLDER/$USERNOW | grep -w CREDITS | awk -F" " '{print $2}')"
                  AFTERCREDS="$(expr "$BEFORECREDS" \- "$CREDITLOSSKB")"
                  sed -e "s/^CREDITS.*/CREDITS $AFTERCREDS/" $USERFOLDER/$USERNOW > $TEMPPATH/$USERNOW.tmp
                  mv -f $TEMPPATH/$USERNOW.tmp $USERFOLDER/$USERNOW
                fi
              fi
              if [ "$TAKE1LOGIN" = "TRUE" ]; then
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "Removing 1 login from $USERNOW"
                fi
                BEFORELOGINS=$(cat $USERFOLDER/$USERNOW | grep -w LOGINS | awk -F" " '{print $2}')
                if [ "$BEFORELOGINS" = "1" ]; then
                  if [ "$DEBUG" = "TRUE" ]; then
                    echo "$USERNOW only have 1 login left. Not removing any."
                  fi
                  if [ "$LOG" != "" ]; then
                    echo "$(date +%x' '%T): DUALDL: $USERNOW looses 1 login. Had 1 already. Not touching." >> $LOG
                  fi
                else
                  if [ "$TEST" != "TRUE" ]; then
                    AFTERLOGINS="$( expr "$BEFORELOGINS" \- "1" )"
                    sed -e "s/^LOGINS $BEFORELOGINS/LOGINS $AFTERLOGINS/" $USERFOLDER/$USERNOW > $TEMPPATH/$USERNOW.tmp
                    mv -f $TEMPPATH/$USERNOW.tmp $USERFOLDER/$USERNOW
                  fi
                fi
              fi
              if [ "$TAKE1DL" = "TRUE" ]; then
                if [ "$LOG" != "" ]; then
                  echo "$(date +%x' '%T): DUALDL: $USERNOW lost 1 download slot." >> $LOG
                fi
                AFTERDL=""
                TMP1="$(cat $USERFOLDER/$USERNOW | grep -w LOGINS | awk -F" " '{print $2}')"
                TMP2="$(cat $USERFOLDER/$USERNOW | grep -w LOGINS | awk -F" " '{print $3}')"
                BEFOREDL="$(cat $USERFOLDER/$USERNOW | grep -w LOGINS | awk -F" " '{print $4}')"
                if [ "$BEFOREDL" = "-1" ]; then
                  if [ "$DEBUG" = "TRUE" ]; then
                    echo "Taking one download. User had unlimited so setting it to 1 instead"
                  fi
                  if [ "$LOG" != "" ]; then
                    echo "$(date +%x' '%T): DUALDL: $USERNOW lost 1 download slot. Had -1, setting to 1." >> $LOG
                  fi
                  AFTERDL="1"
                  OLDSTRING="$TMP1 $TMP2 $BEFOREDL"
                  NEWSTRING="$TMP1 $TMP2 $AFTERDL"
                  if [ "$TEST" != "TRUE" ]; then
                    sed -e "s/^LOGINS $OLDSTRING/LOGINS $NEWSTRING/" $USERFOLDER/$USERNOW > $TEMPPATH/$USERNOW.tmp
                    mv -f $TEMPPATH/$USERNOW.tmp $USERFOLDER/$USERNOW
                  fi
                else
                  if [ "$BEFOREDL" = "1" ]; then
                    if [ "$DEBUG" = "TRUE" ]; then
                      echo "$USERNOW already have 1 sim down (weird since he did dual dl). Wont touch him."
                    fi
                    if [ "$LOG" != "" ]; then
                      echo "$(date +%x' '%T): DUALDL: $USERNOW looses 1 max_sim_down. Already had 1. Not touching." >> $LOG
                    fi
                  else
                    AFTERDL="$( expr "$BEFOREDL" \- "1" )"
                    if [ "$DEBUG" = "TRUE" ]; then
                      echo "Taking 1 max_sim_down from $USERNOW ( from $BEFOREDL to $AFTERDL )."
                    fi
                    if [ "$LOG" != "" ]; then
                      echo "$(date +%x' '%T): DUALDL: $USERNOW lost 1 max_sim_down. From $BEFOREDL to $AFTERDL." >> $LOG
                    fi
                    OLDSTRING="$TMP1 $TMP2 $BEFOREDL"
                    NEWSTRING="$TMP1 $TMP2 $AFTERDL"
                    if [ "$TEST" != "TRUE" ]; then
                      sed -e "s/^LOGINS $OLDSTRING/LOGINS $NEWSTRING/" $USERFOLDER/$USERNOW > $TEMPPATH/$USERNOW.tmp
                      mv -f $TEMPPATH/$USERNOW.tmp $USERFOLDER/$USERNOW
                    fi
                  fi
                fi
              fi

              if [ "$BANDUAL" = "TRUE" ]; then
                if [ "$LASTBAN" != "$USERNOW" ]; then
                  if [ "$DEBUG" = "TRUE" ]; then
                    echo "Banning $USERNOW for $DUALBANTIME seconds."
                  fi
                  if [ "$LOG" != "" ]; then
                    echo "$(date +%x' '%T): DUALDL: $USERNOW - Tempbanning for $DUALBANTIME seconds." >> $LOG
                  fi
                  if [ "$TEST" != "TRUE" ]; then
                  FLAGS="$(cat $USERFOLDER/$USERNOW | grep -w FLAGS | awk -F" " '{print $2}')"
                  CHECK="$( echo $FLAGS | grep 6 )"
                    if [ "$CHECK" != "" ]; then
                      CHECK=""
                      if [ "$DEBUG" = "TRUE" ]; then
                        echo "$USERNOW - Already deleted. Will not delete again."
                        GOTBAN="$USERNOW $GOTBAN"
                      fi
                    else
                      if [ -e $USERFOLDER/$USERNOW ]; then
                        if [ "$DEBUG" = "TRUE" ]; then
                          echo "Found user in $USERFOLDER/$USERNOW. Copying to $TEMPPATH"
                        fi
                        if [ "$TEST" != "TRUE" ]; then
                          cp -f $USERFOLDER/$USERNOW $TEMPPATH/$USERNOW
                          FLAGS="$(cat $USERFOLDER/$USERNOW | grep -w FLAGS | awk -F" " '{print $2}')"
                          sed -e "s/^FLAGS $FLAGS.*/FLAGS "$FLAGS"6/" $USERFOLDER/$USERNOW > $TEMPPATH/$USERNOW.new
                          mv -f $TEMPPATH/$USERNOW.new $USERFOLDER/$USERNOW
                          GOTBAN="$USERNOW $GOTBAN"
                          echo "You were dual downloading. $DUALBANTIME seconds tempban." > $BYEFOLDER/$USERNOW.bye
                        fi
                      fi
                    fi
                  fi
                  LASTBAN=$USERNOW
                fi
              fi

              DUALS2="$USERNOW $DUALS2"
              PIDS="$USERPID $LASTPID $PIDS"

              if [ "$BANDUAL" = "TRUE" ]; then
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "Ban mode for DualDL is TRUE. Will check all pids of this user."
                fi
                for pids in `$WHOBIN | tr -d ' ' | grep -w $user | egrep -v $EXCLUDE1 | sort ` ;do
                  USERPID="$( echo $pids | awk -F"^" '{print $2}')"
                  PIDS2="$USERPID $PIDS2"
                done
                if [ "$PIDS2" = "$PIDS" ]; then
                  if [ "$DEBUG" = "TRUE" ]; then
                    echo "Did not find that user doing anything else on site (but download)."
                  fi
                else
                  if [ "$DEBUG" = "TRUE" ]; then
                    echo "Found the user doing some other stuff. Those will be kicked too."
                  fi
                  PIDS=$PIDS2
                fi
              fi
     
              if [ "$DEBUG" = "TRUE" ]; then
                echo "Going to kick $user - $PIDS"
              fi
              for pid in $PIDS;do
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "killing $pid now"
                  echo "---------------------"
                fi
                if [ "$TEST" != "TRUE" ]; then
                  kill $pid
                fi
              done
            else
              if [ "$DEBUG" = "TRUE" ]; then
                echo "Also killing $USERPID now. Triple downloading?"
              fi
              if [ "$TEST" != "TRUE" ]; then
                kill $USERPID
              fi
            fi
          else
            PIDS=""
            PIDS2=""
          fi
          LASTPID=$USERPID
          LAST=$USERNOW
        done
      done
    fi

    if [ "$GOTBAN" != "" ]; then
      if [ "$DEBUG" = "TRUE" ]; then
        echo "Waiting $DUALBANTIME seconds to readd banned dual download users."
      fi
      sleep $DUALBANTIME
      for user in $GOTBAN; do
        if [ -e $TEMPPATH/$user ]; then
          if [ -e $BYEFOLDER/$user.bye ]; then
            if [ "$TEST" != "TRUE" ]; then
              rm -f $BYEFOLDER/$user.bye
            fi
          fi
          mv -f $TEMPPATH/$user $USERFOLDER/$user
          if [ "$DEBUG" = "TRUE" ]; then
            echo "Restoring $user to $USERFOLDER/$user"
          fi
          if [ "$LOG" != "" ]; then
            echo "$(date +%x' '%T): DUALDL: Restoring $user from tempban." >> $LOG
          fi
        else
          if [ "$DEBUG" = "TRUE" ]; then
            echo "Could not find $TEMPPATH/$user to restore. Guess I didnt delete him."
          fi
        fi
      done
    fi
    LAST=""
    DUALS=""
    DUALS2=""
    LASTPID=""
    USERPID=""
    USERNOW=""
    u=""
    user="" 
  fi
fi

#####################################################################################
## Dual Uploaders part.                                                             #
#####################################################################################

if [ "$DUALUP" = "TRUE" ]; then
  if [ "$DEBUG" = "TRUE" ]; then
    echo ""
    echo "-----------------------------------"
    echo "Start Dual UP kick part.           "
  fi

  LAST=""
  DUALS=""
  DUALS2=""

  if [ "$KILLGHOST" != "" ]; then
    $KILLGHOST
  fi

  USERSON="0"
  for online in `$WHOBIN | tr -d ' '`; do
    USERSON="$( expr "$USERSON" \+ "1" )"
  done

  if [ "$USERSON" -lt "$LIMITTODUALUPKICK" ]; then
    if [ "$DEBUG" = "TRUE" ]; then
      echo "Only $USERSON users online. Not running unless theres $LIMITTODUALUPKICK on."
    fi
    DUALUP="NO"
  else
    DUALUP="YES"
  fi 

  if [ "$DEBUG" = "TRUE" ]; then
    if [ "$SHOWEXCLUDES" = "TRUE" ]; then
      if [ "$EXCLUDE4" != "" ]; then
        EXCLUDE4VIEW="$( echo $EXCLUDE4 | tr -s '|' ' ' )"
        echo " "
        echo "Based on excludes from groups and users, the following users wont be checked:"
        echo "$EXCLUDE4VIEW"
        echo " "
      fi
    fi
  fi

  if [ "$DUALUP" = "YES" ]; then
    for u in `$WHOBIN | tr -d ' ' | grep -w Up: | egrep -v $EXCLUDE4 | sort ` ;do
      USERNOW="$( echo $u | awk -F"^" '{print $1}')"
      if [ "$PREFOLDERS" != "" ]; then
        CHECKING="$( echo $PREUSERS | grep -w $USERNOW )"
        if [ "$CHECKING" = "" ]; then
          for prefolder in $PREFOLDERS; do
            VERPREFOLDER="$( echo $u | grep -w $prefolder )"
            if [ "$VERPREFOLDER" != "" ]; then
              if [ "$DEBUG" = "TRUE" ]; then
                echo "$USERNOW is in $prefolder and wont be checked"
              fi
              INPREFOLDERDL="YES"
              PREUSERS="$USERNOW $PREUSERS"
            fi
          done
        fi
        VERPREFOLDER=""
        prefolder=""
      fi
      if [ "$USERNOW" != "$LAST" ]; then
        UPFOLDER="$( echo $u | awk -F"^" '{print $5}' )"
        UPFOLDER="$( echo $UPFOLDER | tr -s '/' ' ' )"
      fi

      if [ "$CHECKING" != "" ]; then
        INPREFOLDERDL="YES"
      fi

      if [ "$INPREFOLDERDL" != "YES" ]; then
        if [ -e "$USERFOLDER/$USERNOW" ]; then
          ok=yes
        else
          USERNOW="$RANDOM"
        fi
        if [ "$USERNOW" = "$LAST" ]; then
          UPFOLDER2="$( echo $u | awk -F"^" '{print $5}' )"
          UPFOLDER2="$( echo $UPFOLDER2 | tr -s '/' ' ' )"
  
          UPFOLDERNUMBER="$( echo "$UPFOLDER" | wc -w )"
          UPTO="0"
          for each in $UPFOLDER; do
            UPTO="$( expr $UPTO \+ 1 )"
            if [ "$UPTO" = "1" ]; then
              somethinge=else
            else
              if [ "$UPTO" -lt "$UPFOLDERNUMBER" ]; then
                NEWUP="$NEWUP $each"
              fi
            fi
          done
          NEWUP="$( echo $NEWUP | tr -s ' ' '/' )"

          UPFOLDERNUMBER2="$( echo "$UPFOLDER2" | wc -w )"
          UPTO="0"
          for each in $UPFOLDER2; do
            UPTO="$( expr $UPTO \+ 1 )"
            if [ "$UPTO" = "1" ]; then
              somethinge=else
            else
              if [ "$UPTO" -lt "$UPFOLDERNUMBER2" ]; then
                NEWUP2="$NEWUP2 $each"
              fi
            fi
          done
          NEWUP2="$( echo $NEWUP2 | tr -s ' ' '/' )"

          if [ "$NEWUP" = "$NEWUP2" ]; then
            if [ "$DEBUG" = "TRUE" ]; then
              echo "$USERNOW is dual uploading in $NEWUP2"
            fi
            if [ "$BOTDUALUPW" = "TRUE" ]; then
              echo "$(date +'%a %b %d %T %Y') $BOTTRIGGER \002-(DUALUP)- $USERNOW\002, early warning for dual upload in /$NEWUP. Checking again in $DELAYDUALUP seconds." >> $GLLOG
            fi
            if [ "$LOG" != "" ]; then
              echo "$(date +%x' '%T): DUALUP: $USERNOW - Early warning. Dual up in /$NEWUP" >> $LOG
            fi
            DUALUPS="$USERNOW $DUALUPS"
          fi
        fi
        LAST="$USERNOW"
      fi
    done
  fi

  if [ "$DUALUPS" = "" ]; then
    if [ "$DEBUG" = "TRUE" ]; then
      echo "No users dual upping to the same release. Quitting Dual Upload kicker part"
      if [ -e $TEMPPATH/ktmfdualup.tmp ]; then
        rm -rf $TEMPPATH/ktmfdualup.tmp
      fi
      NODUALUP="TRUE"
    fi
  else
    if [ "$DEBUG" = "TRUE" ]; then
      echo "Going to check $DUALUPS after $DELAYDUALUP seconds."
    fi
  fi

  PIDS=""
  VERIFY=""
  USERNOW=""
  USERPID=""
  VERIFY=""
  NEWUP=""
  NEWUP2=""
  LAST=""
  
  if [ "$NODUALUP" != "TRUE" ]; then
    sleep $DELAYDUALUP
    LAST=""
    USERNOW=""

    if [ "$DEBUG" = "TRUE" ]; then
      echo "-------------------------"
    fi

    for user in $DUALUPS; do
      for u in `$WHOBIN | tr -d ' ' | grep -w Up: | grep -w $user | egrep -v EXCLUDE4 | sort ` ;do
        USERNOW="$( echo $u | awk -F"^" '{print $1}')"
        DUALUPPIDS="$USERPID $DUALUPPIDS"        
        if [ "$USERNOW" != "$LAST" ]; then
          UPFOLDER="$( echo $u | awk -F"^" '{print $5}' )"
          UPFOLDER="$( echo $UPFOLDER | tr -s '/' ' ' )"
        fi
        if [ "$USERNOW" = "$LAST" ]; then
          UPFOLDER2="$( echo $u | awk -F"^" '{print $5}' )"
          UPFOLDER2="$( echo $UPFOLDER2 | tr -s '/' ' ' )"
          UPFOLDERNUMBER="$( echo "$UPFOLDER2" | wc -w )"
          UPTO="0"
          for each in $UPFOLDER; do
            UPTO="$( expr $UPTO \+ 1 )"
            if [ "$UPTO" = "1" ]; then
              somethinge=else
            else
              if [ "$UPTO" -lt "$UPFOLDERNUMBER" ]; then
                NEWUP="$NEWUP $each"
              fi
            fi
          done
          NEWUP="$( echo $NEWUP | tr -s ' ' '/' )"

          UPFOLDERNUMBER2="$( echo "$UPFOLDER2" | wc -w )"
          UPTO="0"
          for each in $UPFOLDER2; do
            UPTO="$( expr $UPTO \+ 1 )"
            if [ "$UPTO" = "1" ]; then
              somethinge=else
            else
              if [ "$UPTO" -lt "$UPFOLDERNUMBER2" ]; then
                NEWUP2="$NEWUP2 $each"
              fi
            fi
          done
          NEWUP2="$( echo $NEWUP2 | tr -s ' ' '/' )"

          if [ "$NEWUP" = "$NEWUP2" ]; then
            if [ "$DEBUG" = "TRUE" ]; then
              echo "$USERNOW is still dual uploading in $NEWUP2"
            fi
            if [ "$LOG" != "" ]; then
              echo "$(date +%x' '%T): DUALUP: $USERNOW kicked for dual up in /$NEWUP" >> $LOG
            fi
            if [ "$BOTDUALUP" = "TRUE" ]; then
              echo "$(date +'%a %b %d %T %Y') $BOTTRIGGER \002-(DUALUP)- $USERNOW\002 kicked for dual uploading in /$NEWUP." >> $GLLOG
            fi
            DUALUPSKICK="$USERNOW $DUALUPSKICK"
          fi
          INPREFOLDERDL=""
        fi 
        LAST="$USERNOW"
        NEWUP=""
        NEWUP2=""
      done
    done

    if [ "$DUALUPSKICK" != "" ]; then
      DUALUPPIDS=""
      for userkick in $DUALUPSKICK; do
        for u in `$WHOBIN | tr -d ' ' | grep -w Up: | grep -w $userkick | sort ` ;do
          USERPID="$( echo $u | awk -F"^" '{print $2}')"
          DUALUPPIDS="$USERPID $DUALUPPIDS"
        done
      done
       
      if [ "$DEBUG" = "TRUE" ]; then
        echo "Punishing users $DUALUPSKICK"
      fi

      if [ "$CREDITLOSSDU" = "TRUE" ]; then
        for loser in $DUALUPSKICK; do
          if [ "$DEBUG" = "TRUE" ]; then
            echo "Taking $CREDITLOSSDUMB MB from $loser"
          fi
          if [ "$LOG" != "" ]; then
            echo "$(date +%x' '%T): DUALUP: $loser lost $CREDITLOSSDUMB MB" >> $LOG
          fi
          if [ "$TEST" != "TRUE" ]; then
            CREDITLOSSKB="$( expr "$CREDITLOSSDUMB" \* "1024" )"
            BEFORECREDS="$(cat $USERFOLDER/$loser | grep -w CREDITS | awk -F" " '{print $2}')"
            AFTERCREDS="$(expr "$BEFORECREDS" \- "$CREDITLOSSKB")" 
            sed -e "s/^CREDITS.*/CREDITS $AFTERCREDS/" $USERFOLDER/$loser > $TEMPPATH/$loser.tmp
            mv -f $TEMPPATH/$loser.tmp $USERFOLDER/$loser
          fi
        done
      fi

      if [ "$TAKE1LOGINDU" = "TRUE" ]; then
        for loser in $DUALUPSKICK; do
          if [ "$DEBUG" = "TRUE" ]; then
            echo "Removing 1 login from $loser"
          fi
          if [ "$LOG" != "" ]; then
            echo "$(date +%x' '%T): DUALUP: $loser lost 1 login slot." >> $LOG
          fi
          BEFORELOGINS=$(cat $USERFOLDER/$loser | grep -w LOGINS | awk -F" " '{print $2}')
          if [ "$BEFORELOGINS" = "1" ]; then
            if [ "$DEBUG" = "TRUE" ]; then
              echo "$loser only have 1 login left. Not removing any."
            fi
          else
            if [ "$TEST" != "TRUE" ]; then
              AFTERLOGINS="$( expr "$BEFORELOGINS" \- "1" )"
              sed -e "s/^LOGINS $BEFORELOGINS/LOGINS $AFTERLOGINS/" $USERFOLDER/$loser > $TEMPPATH/$loser.tmp
              mv -f $TEMPPATH/$loser.tmp $USERFOLDER/$loser
            fi
          fi
        done
      fi

      if [ "$TAKE1UP" = "TRUE" ]; then
        for loser in $DUALUPSKICK; do
          AFTERUP=""
          TMP1="$(cat $USERFOLDER/$loser | grep -w LOGINS | awk -F" " '{print $2}')"
          TMP2="$(cat $USERFOLDER/$loser | grep -w LOGINS | awk -F" " '{print $3}')"
          TMP3="$(cat $USERFOLDER/$loser | grep -w LOGINS | awk -F" " '{print $4}')"
          BEFOREUP="$(cat $USERFOLDER/$loser | grep -w LOGINS | awk -F" " '{print $5}')"

          if [ "$BEFOREUP" = "-1" ]; then
            if [ "$DEBUG" = "TRUE" ]; then
              echo "Taking one upload from $loser. User had unlimited so setting it to 1 instead"
            fi
            if [ "$LOG" != "" ]; then
              echo "$(date +%x' '%T): $loser looses 1 upload slot. Had unlimited. Setting 1." >> $LOG
            fi
            AFTERUP="1"
            OLDSTRING="$TMP1 $TMP2 $TMP3 $BEFOREUP"
            NEWSTRING="$TMP1 $TMP2 $TMP3 $AFTERUP"
            if [ "$TEST" != "TRUE" ]; then
              sed -e "s/^LOGINS $OLDSTRING/LOGINS $NEWSTRING/" $USERFOLDER/$loser > $TEMPPATH/$loser.tmp
              mv -f $TEMPPATH/$loser.tmp $USERFOLDER/$loser
            fi
          else
            if [ "$BEFOREUP" = "1" ]; then
               if [ "$DEBUG" = "TRUE" ]; then
                 echo "$loser already have 1 sim up (weird since he did dual up). Wont touch him."
               fi
               if [ "$LOG" != "" ]; then
                 echo "$(date +%x' '%T): DUALUP: $loser looses 1 upload slot. Already had 1 (weird)." >> $LOG
               fi
            else
              AFTERUP="$( expr "$BEFOREUP" \- "1" )"
              if [ "$DEBUG" = "TRUE" ]; then
                echo "Taking 1 max_sim_up from $loser ( from $BEFOREUP to $AFTERUP )."
              fi
              if [ "$LOG" != "" ]; then
                echo "$(date +%x' '%T): DUALUP: $loser looses 1 upload slot. From $BEFOREUP to $AFTERUP." >> $LOG
              fi
              OLDSTRING="$TMP1 $TMP2 $TMP3 $BEFOREUP"
              NEWSTRING="$TMP1 $TMP2 $TMP3 $AFTERUP"
              if [ "$TEST" != "TRUE" ]; then
                sed -e "s/^LOGINS $OLDSTRING/LOGINS $NEWSTRING/" $USERFOLDER/$loser > $TEMPPATH/$loser.tmp
                mv -f $TEMPPATH/$loser.tmp $USERFOLDER/$loser
              fi
            fi
          fi
        done
      fi

      GOTBAN=""
      if [ "$BANDUALUP" = "TRUE" ]; then
        for loser in $DUALUPSKICK; do
          if [ "$LASTBAN" != "$loser" ]; then
            if [ "$DEBUG" = "TRUE" ]; then
              echo "Banning $loser for $DUALUPBANTIME seconds."
            fi
            if [ "$LOG" != "" ]; then
              echo "$(date +%x' '%T): DUALUP: $loser gets tempbanned for $DUALUPBANTIME seconds." >> $LOG
            fi
            GOTBAN="$loser $GOTBAN"

            if [ "$TEST" != "TRUE" ]; then
              FLAGS="$(cat $USERFOLDER/$loser | grep -w FLAGS | awk -F" " '{print $2}')"
              CHECK="$( echo $FLAGS | grep 6 )"
              if [ "$CHECK" != "" ]; then
                CHECK=""
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "$loser - Already deleted. Will not delete again."
                fi
              else
                if [ -e $USERFOLDER/$loser ]; then
                  if [ "$DEBUG" = "TRUE" ]; then
                    echo "Found user in $USERFOLDER/$loser. Copying to $TEMPPATH"
                  fi
                  if [ "$TEST" != "TRUE" ]; then
                    cp -f $USERFOLDER/$loser $TEMPPATH/$loser
                    FLAGS="$(cat $USERFOLDER/$loser | grep -w FLAGS | awk -F" " '{print $2}')"
                    sed -e "s/^FLAGS $FLAGS.*/FLAGS "$FLAGS"6/" $USERFOLDER/$loser > $TEMPPATH/$loser.new
                    mv -f $TEMPPATH/$loser.new $USERFOLDER/$loser
                    GOTBAN="$loser $GOTBAN"
                    echo "You were dual uploading on the same release. $DUALUPBANTIME seconds tempban." > $BYEFOLDER/$loser.bye
                  fi
                fi
              fi
            fi
            LASTBAN=$loser
          fi

          PIDS2=""
          if [ "$DEBUG" = "TRUE" ]; then
            if [ "$SAIDIT" != "YES" ]; then
              echo "Ban mode for DualUP is TRUE. Will check all pids of this user."
              SAIDIT="YES"
            fi
          fi
          for pids in `$WHOBIN | tr -d ' ' | grep -w $loser | egrep -v $EXCLUDE4 | sort ` ;do
            USERPID="$( echo $pids | awk -F"^" '{print $2}')"
            PIDS2="$USERPID $PIDS2"
          done
          #echo "Pids for this user: $PIDS2"
          for morepid in $PIDS2; do
            CHECK="$( echo $DUALUPPIDS | grep -w $morepid )"
            if [ "$CHECK" = "" ]; then
              FOUND="YES"
            fi
          done
          if [ "$FOUND" != "YES" ]; then
            if [ "$DEBUG" = "TRUE" ]; then
              echo "Did not find $loser doing anything else on site (but those uploads)."
            fi
          else
            if [ "$DEBUG" = "TRUE" ]; then
              echo "Found $loser doing some other stuff. Those will be kicked too."
            fi
            USERPID=""
          fi
          DUALUPPIDS2="$DUALUPPIDS2 $PIDS2"
          FOUND="NO"
          PIDS2=""
        done
        DUALUPPIDS="$DUALUPPIDS2"
      fi

      if [ "$DUALUPPIDS" != "" ]; then
        if [ "$DEBUG" = "TRUE" ]; then
          echo "Pids to kick: $DUALUPPIDS"
        fi
        for pid in $DUALUPPIDS; do
          if [ "$DEBUG" = "TRUE" ]; then
            echo "Kicking $pid now."
          fi
          if [ "$TEST" != "TRUE" ]; then
            kill $pid
          fi
        done
      fi

      if [ "$GOTBAN" != "" ]; then
        if [ "$DEBUG" = "TRUE" ]; then
          echo "Waiting $DUALUPBANTIME seconds to readd banned dual upload users."
        fi
        sleep $DUALUPBANTIME
        for user in $DUALUPSKICK; do
          if [ -e $TEMPPATH/$user ]; then
            if [ -e $BYEFOLDER/$user.bye ]; then
              if [ "$TEST" != "TRUE" ]; then
                rm -f $BYEFOLDER/$user.bye
              fi
            fi
            mv -f $TEMPPATH/$user $USERFOLDER/$user
            if [ "$DEBUG" = "TRUE" ]; then
              echo "Restoring $user to $USERFOLDER/$user"
            fi
            if [ "$LOG" != "" ]; then
              echo "$(date +%x' '%T): DUALUP: Restoring $user from tempban." >> $LOG
            fi
          else
            if [ "$DEBUG" = "TRUE" ]; then
              echo "Could not find $TEMPPATH/$user to restore. Guess I didnt delete him."
            fi
          fi
        done
      fi
   
    fi 
  fi    
  user=""
  USERNOW=""
  pid=""
  PIDS=""
  GOTBAN=""
  PIDS2=""
  VERIFY=""
fi



#####################################################################################
## Idle killer part.                                                                #
#####################################################################################

if [ "$IDLEKILL" = "TRUE" ]; then
  if [ "$DEBUG" = "TRUE" ]; then
    echo " "
    echo "-----------------------------------"
    echo "Start Idle kick part.           "
  fi

  if [ "$KILLGHOST" != "" ]; then
    $KILLGHOST
  fi

  USERSON="0"
  for online in `$WHOBIN | tr -d ' '`; do
    USERSON="$( expr "$USERSON" \+ "1" )"
  done

  if [ "$USERSON" -lt "$LIMITTOIDLEKICK" ]; then
    if [ "$DEBUG" = "TRUE" ]; then
      echo "Only $USERSON users online. Not running unless theres $LIMITTOIDLEKICK on."
    fi
    IDLEGO="NO"
  else
    IDLEGO="YES"
  fi 

  if [ "$DEBUG" = "TRUE" ]; then
    if [ "$SHOWEXCLUDES" = "TRUE" ]; then
      if [ "$EXCLUDE2" != "" ]; then
      EXCLUDE2VIEW="$( echo $EXCLUDE2 | tr -s '|' ' ' )"
      echo " "
      echo "Based on excludes from groups and users, the following users wont be checked:"
      echo "$EXCLUDE2VIEW"
      echo " "
      fi
    fi
  fi

  LAST=""
  IDLES=""
  IDLES2=""
  VERIFY=""
  GOTBAN=""

  if [ "$IDLEGO" = "YES" ]; then
    for u in `$WHOBIN | tr -d ' ' | grep -w Idle: | egrep -v $EXCLUDE2 | sort ` ;do
      USERNOW="$( echo $u | awk -F"^" '{print $1}')"
      if [ "$PREFOLDERS" != "" ]; then
        for prefolder in $PREFOLDERS; do
          VERPREFOLDER="$( echo $u | grep -w $prefolder )"
          if [ "$VERPREFOLDER" != "" ]; then
            if [ "$DEBUG" = "TRUE" ]; then
              echo "$USERNOW: idle in $prefolder. Wont be checked."
            fi
            IDLEBEFORE="$( echo $IDLES | grep -w $USERNOW )"
            if [ "$IDLEBEFORE" != "" ]; then
              NEWIDLES=""
              for idlel in $IDLES; do
                rebuildcheck="$( echo $idlel | grep -w $USERNOW )"
                if [ "$rebuildcheck" = "" ]; then
                  NEWIDLES="$idlel $NEWIDLES"
                fi
              done
              IDLES="$NEWIDLES"
              NEWIDLES=""
            fi
            INPREFOLDERS="$USERNOW $INPREFOLDERS"
          fi
        done
        VERPREFOLDER=""
        prefolder=""
      fi

      ALREADYEXCLUDED="$( echo $INPREFOLDERS | grep -w $USERNOW )"
      if [ "$ALREADYEXCLUDED" = "" ]; then
        VERIFY="$( echo $IDLES | grep $USERNOW )"
        if [ -e "$USERFOLDER/$USERNOW" ]; then
          ok=yes
        else
          VERIFY="$RANDOM"
        fi
        if [ "$VERIFY" = "" ]; then
          if [ "$BOTIDLEW" = "TRUE" ]; then
            echo "$(date +'%a %b %d %T %Y') $BOTTRIGGER \002-(IDLE)- $USERNOW\002, early warning for idling. Checking again in $DELAYIDLEKILL seconds." >> $GLLOG
            if [ "$LOG" != "" ]; then
              echo "$(date +%x' '%T): IDLE: $USERNOW got an early warning for idling." >> $LOG
            fi
          fi
          if [ "$DEBUG" = "TRUE" ]; then
            echo "$USERNOW is idle."
          fi
          IDLES="$USERNOW $IDLES"
        fi
      fi
      ALREADYEXCLUDED=""
      LAST=$USERNOW
    done

    if [ "$IDLES" = "" ]; then
      if [ "$DEBUG" = "TRUE" ]; then
        echo "No users idle. Quitting Idle kicker part"
        if [ -e $TEMPPATH/ktmfidle.tmp ]; then
          rm -rf $TEMPPATH/ktmfidle.tmp
        fi
        NOIDLE="TRUE"
      fi
    else
      if [ "$DEBUG" = "TRUE" ]; then
        echo "Going to check $IDLES after $DELAYIDLEKILL seconds."
      fi
    fi

    PIDS=""
    PIDS2=""
    VERIFY=""
    GOTBAN=""

    if [ "$NOIDLE" != "TRUE" ]; then
      sleep $DELAYIDLEKILL
      LAST=""
      USERNOW=""

      if [ "$DEBUG" = "TRUE" ]; then
        echo "-------------------------"
      fi

      for user in `echo $IDLES`; do
        for u in `$WHOBIN | tr -d ' ' | grep -w Idle: | grep -w $user | egrep -v $EXCLUDE2 | sort ` ;do
          USERNOW="$( echo $u | awk -F"^" '{print $1}')"
          USERPID="$( echo $u | awk -F"^" '{print $2}')"
          if [ "$USERNOW" != "$LAST" ]; then
            VERIFY="$( echo $IDLES2 | grep $USERNOW )"
            if [ "$VERIFY" = "" ]; then
              if [ "$BOTIDLE" = "TRUE" ]; then
                echo "$(date +'%a %b %d %T %Y') $BOTTRIGGER \002-(IDLE)- $USERNOW\002 kicked for idling!" >> $GLLOG
              fi
              if [ "$DEBUG" = "TRUE" ]; then
                echo "$USERNOW is still idle"
              fi
              if [ "$LOG" != "" ]; then
                echo "$(date +%x' '%T): IDLE: $USERNOW kicked for idling." >> $LOG
              fi
              touch $TEMPPATH/ktmfidle.tmp
              OLDIDLELISTVER="$(cat $TEMPPATH/ktmfidle.tmp | grep -w $user | awk -F" " '{print $1}')"
              if [ "$OLDIDLELISTVER" = "" ]; then
                echo "$user 1" >> $TEMPPATH/ktmfidle.tmp
                TIMESIDLE="1"
              else
                TIMESIDLE="$(cat $TEMPPATH/ktmfidle.tmp | grep -w $user | awk -F" " '{print $2}')"
                IDLENOW="$( expr "$TIMESIDLE" \+ "1" )"
                sed -e "s/^$user.*/$user $IDLENOW/" $TEMPPATH/ktmfidle.tmp > $TEMPPATH/idles.tmp
                mv -f $TEMPPATH/idles.tmp $TEMPPATH/ktmfidle.tmp
              fi
              IDLEUSERSKICKED="$user $IDLEUSERSKICKED"
              USERNOW=$user
              if [ "$CREDITLOSSIK" = "TRUE" ]; then
                if [ "$TIMESIDLE" -lt $TIMESBEFOREBANI ]; then
                  if [ "$DEBUG" = "TRUE" ]; then
                    echo "$user has not yet idled enough times to loose creds."
                  fi
                  if [ "$LOG" != "" ]; then
                    echo "$(date +%x' '%T): IDLE: $USERNOW hasnt idled enough times to loose creds." >> $LOG
                  fi
                else
                  if [ "$DEBUG" = "TRUE" ]; then
                    echo "Taking $CREDITLOSSIKMB MB from $USERNOW"
                  fi
                  if [ "$LOG" != "" ]; then
                    echo "$(date +%x' '%T): IDLE: $USERNOW lost $CREDITLOSSIKMB MB credits." >> $LOG
                  fi
                  if [ "$TEST" != "TRUE" ]; then
                    CREDITLOSSKB="$( expr "$CREDITLOSSIKMB" \* "1024" )"
                    BEFORECREDS="$(cat $USERFOLDER/$USERNOW | grep -w CREDITS | awk -F" " '{print $2}')"
                    AFTERCREDS="$(expr "$BEFORECREDS" \- "$CREDITLOSSKB")"
                    sed -e "s/^CREDITS.*/CREDITS $AFTERCREDS/" $USERFOLDER/$USERNOW > $TEMPPATH/$USERNOW.tmp
                    mv -f $TEMPPATH/$USERNOW.tmp $USERFOLDER/$USERNOW
                  fi
                fi
              fi

              if [ "$BANIDLE" = "TRUE" ]; then
                if [ "$TIMESIDLE" -lt $TIMESTOBAN ]; then
                  if [ "$DEBUG" = "TRUE" ]; then
                    echo "$user has not yet idled enough times to be banned."
                  fi
                  if [ "$LOG" != "" ]; then
                    echo "$(date +%x' '%T): IDLE: $user has not idled enough times to be tempbanned." >> $LOG
                  fi
                else
                  if [ "$DEBUG" = "TRUE" ]; then
                    echo "Banning $USERNOW for idling more then $TIMESTOBAN times."
                  fi
                  if [ "$LOG" != "" ]; then
                    echo "$(date +%x' '%T): IDLE: $USERNOW gets tempbanned for $IDLEBANTIME secs. Idle $TIMESTOBAN times in a row." >> $LOG
                  fi
                  if [ "$TEST" != "TRUE" ]; then
                    if [ "$LASTBAN" != "$USERNOW" ]; then
                      if [ "$DEBUG" = "TRUE" ]; then
                        echo "Banning $USERNOW for $IDLEBANTIME seconds."
                      fi
                      if [ "$TEST" != "TRUE" ]; then
                        FLAGS="$(cat $USERFOLDER/$USERNOW | grep -w FLAGS | awk -F" " '{print $2}')"
                        CHECK="$( echo $FLAGS | grep 6 )"
                        if [ "$CHECK" != "" ]; then
                          CHECK=""
                          if [ "$DEBUG" = "TRUE" ]; then
                            echo "$USERNOW - Already deleted. Will not delete again."
                          fi
                          GOTBAN="$USERNOW $GOTBAN"
                        else
                          if [ -e $USERFOLDER/$USERNOW ]; then
                            if [ "$DEBUG" = "TRUE" ]; then
                              echo "Found user in $USERFOLDER/$USERNOW. Copying to $TEMPPATH"
                            fi
                            cp -f $USERFOLDER/$USERNOW $TEMPPATH/$USERNOW
                            FLAGS="$(cat $USERFOLDER/$USERNOW | grep -w FLAGS | awk -F" " '{print $2}')"
                            sed -e "s/^FLAGS $FLAGS.*/FLAGS "$FLAGS"6/" $USERFOLDER/$USERNOW > $TEMPPATH/$USERNOW.new
                            mv -f $TEMPPATH/$USERNOW.new $USERFOLDER/$USERNOW
                            GOTBAN="$USERNOW $GOTBAN"
                            echo "You have idled $TIMESIDLE times in a row. Tempban initiated for $IDLEBANTIME seconds." > $BYEFOLDER/$USERNOW.bye
                          fi
                        fi
                      fi
                    fi
                  fi
                LASTBAN=$USERNOW
                fi
              fi

              PIDS="$USERPID"
              IDLES2="$USERNOW $IDLES2"

              if [ "$BANIDLE" = "TRUE" ]; then
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "Ban mode for Idling is TRUE. Will check all pids of this user."
                fi
                for pids in `$WHOBIN | tr -d ' ' | grep -w $user | egrep -v $EXCLUDE1 | sort ` ;do
                  USERPID="$( echo $pids | awk -F"^" '{print $2}')"
                  PIDS2="$USERPID $PIDS2"
                done
                if [ "$PIDS2" = "$PIDS " ]; then
                  if [ "$DEBUG" = "TRUE" ]; then
                    echo "Did not find that user doing anything else on site (but idling)."
                  fi
                  PIDS2=""
                else
                  if [ "$DEBUG" = "TRUE" ]; then
                    echo "Found the user doing some other stuff. Those will be kicked too."
                  fi
                  PIDS=$PIDS2
                fi
              fi
 
              if [ "$DEBUG" = "TRUE" ]; then

                echo "Going to kick $user - $PIDS"
              fi
              if [ "$TEST" != "TRUE" ]; then
                for pid in $PIDS; do
                  kill $pid
                done
              fi
            else
              if [ "$DEBUG" = "TRUE" ]; then
                echo "Also killing $USERPID now. Triple idler?"
              fi
              if [ "$TEST" != "TRUE" ]; then
                kill $USERPID
              fi
            fi
          else
            p=""
            PIDS2=""
            PIDS=""
            USERPID2=""
          fi
          LASTPID=$USERPID
          LAST=$USERNOW
        done
      done
    fi
    LAST=""
    IDLES=""
    IDLES2=""
    LASTPID=""
    USERPID=""
    USERNOW=""
    u=""
    user=""


    if [ "$IDLEKILL" = "TRUE" ]; then
      for userskick in $IDLEUSERSKICKED; do
        touch $TEMPPATH/ktmfidle.tmp
        OLDIDLELISTVER="$(cat $TEMPPATH/ktmfidle.tmp | grep -w $userskick | awk -F" " '{print $1}')"
        TIMESIDLE="$(cat $TEMPPATH/ktmfidle.tmp | grep -w $userskick | awk -F" " '{print $2}')"
        if [ "$FIRSTIDLE" != "NO" ]; then
          echo "$userskick $TIMESIDLE" > $TEMPPATH/ktmf.tmp
          FIRSTIDLE="NO"  
        else
          echo "$userskick $TIMESIDLE" >> $TEMPPATH/ktmf.tmp
        fi  
      done

      if [ -e $TEMPPATH/ktmf.tmp ]; then 
        mv -f $TEMPPATH/ktmf.tmp $TEMPPATH/ktmfidle.tmp
      else
        rm -f $TEMPPATH/ktmf.tmp
      fi
    fi

    if [ "$GOTBAN" != "" ]; then
      if [ "$DEBUG" = "TRUE" ]; then
        echo "Waiting $IDLEBANTIME seconds to readd banned idle users."
      fi
      sleep $IDLEBANTIME
      for user in $GOTBAN; do
        if [ "$LOG" != "" ]; then
          echo "$(date +%x' '%T): IDLE: Restoring $user from tempban." >> $LOG
        fi
        if [ -e $BYEFOLDER/$user.bye ]; then
          rm -f $BYEFOLDER/$user.bye
        fi
        if [ -e $TEMPPATH/$user ]; then
          mv -f $TEMPPATH/$user $USERFOLDER/$user
          if [ "$DEBUG" = "TRUE" ]; then
            echo "Restoring $user to $USERFOLDER/$user"
          fi
        else
          if [ "$DEBUG" = "TRUE" ]; then
            echo "Could not find $TEMPPATH/$user to restore. Guess I didnt delete him."
          fi
        fi
      done
    fi
  fi
fi


#####################################################################################
## Slow Downloaders part                                                            #
#####################################################################################

GOTBAN=""
PIDS=""
PIDS2=""
USERSON="0"
TOCHECK=""

if [ "$SLOWDLKICK" = "TRUE" ]; then
  if [ "$DEBUG" = "TRUE" ]; then
    echo " "
    echo "-----------------------------------"
    echo "Start Slow download kick part.     "
  fi

  if [ "$KILLGHOST" != "" ]; then
    $KILLGHOST
  fi

  for online in `$WHOBIN | tr -d ' '`; do
    USERSON="$( expr "$USERSON" \+ "1" )"
  done
  
  if [ "$USERSON" -lt "$LIMITTOKICK" ]; then
    if [ "$DEBUG" = "TRUE" ]; then
      echo "Only $USERSON users online. Not running unless theres $LIMITTOKICK on."
    fi
    SLOWDLGO="NO"
  else
    SLOWDLGO="YES"
  fi 

  if [ "$DEBUG" = "TRUE" ]; then
    if [ "$SHOWEXCLUDES" = "TRUE" ]; then
      if [ "$EXCLUDE3" != "" ]; then
        EXCLUDE3VIEW="$( echo $EXCLUDE3 | tr -s '|' ' ' )"
        echo " "
        echo "Based on excludes from groups and users, the following users wont be checked:"
        echo "$EXCLUDE3VIEW"
        echo " "
      fi
    fi
  fi

  if [ "$SLOWDLGO" = "YES" ]; then
    for u in `$WHOBIN | tr -d ' ' | grep -w Dn: | egrep -v $EXCLUDE3 | sort` ;do
      USERNOW="$( echo $u | awk -F"^" '{print $1}')"
      USERPID="$( echo $u | awk -F"^" '{print $2}')"
      USERSPEED="$( echo $u | awk -F"^" '{print $4}')"
      USERSPEED2="$( echo $USERSPEED | awk -F"." '{print $1}')"
      if [ "$USERSPEED2" -lt "$SPEEDLIMIT" ]; then
        
        if [ "$PREFOLDERS" != "" ]; then
          for prefolder in $PREFOLDERS; do
            VERPREFOLDER="$( echo $u | grep -w $prefolder )"
            if [ "$VERPREFOLDER" != "" ]; then
              if [ "$DEBUG" = "TRUE" ]; then
                echo "$USERNOW: Slowdownloading in in $prefolder. Wont be checked."
              fi
              INPREFOLDERSLOW="YES"
              IDLEBEFORE="$( echo $TOCHECK | grep -w $USERNOW )"
              if [ "$IDLEBEFORE" != "" ]; then
                NEWTOCHECK=""
                for slowl in $TOCHECK; do
                  rebuildcheck="$( echo $slowl | grep -w $USERNOW )"
                  if [ "$rebuildcheck" = "" ]; then
                    NEWTOCHECK="$slowl $NEWTOCHECK"
                  fi
                done
                TOCHECK="$NEWTOCHECK"
                NEWTOCHECK=""
              fi
              INPREFOLDERS="$USERNOW $INPREFOLDERS"
            fi
          done
          VERPREFOLDER=""
          prefolder=""
        fi
        
        if [ "$INPREFOLDERSLOW" != "YES" ]; then
          if [ "$DEBUG" = "TRUE" ]; then
            echo "$USERNOW is only downloading with $USERSPEED k/sec"
          fi
          VERIFY="$( echo $TOCHECK | grep -w $USERNOW )"
          if [ -e "$USERFOLDER/$USERNOW" ]; then
            ok=yes
          else
            VERIFY="$RANDOM"
          fi
          if [ "$VERIFY" = "" ]; then
            if [ "$BOTSLOWW" = "TRUE" ]; then
              if [ "$LOG" != "" ]; then
                echo "$(date +%x' '%T): SLOWDL: $USERNOW gets a warning for slow downloading ($USERSPEED k/Sec)" >> $LOG
              fi
              echo "$(date +'%a %b %d %T %Y') $BOTTRIGGER \002-(SLOWDL)- $USERNOW\002, early warning for slow downloading at $USERSPEED k/sec. Rechecking in $DELAYSLOWDL seconds." >> $GLLOG
            fi
            TOCHECK="$USERNOW $TOCHECK"
          fi
        fi
        INPREFOLDERSLOW=""     
      fi
    done
    
  SLOWDLGO=""

    if [ "$TOCHECK" = "" ]; then
      if [ "$DEBUG" = "TRUE" ]; then
        echo "Nobody is downloading below $SPEEDLIMIT k/sec"
      fi
      SLOWDLGO="NO"
    else
      if [ "$DEBUG" = "TRUE" ]; then
        echo "Going to check $TOCHECK after $DELAYSLOWDL seconds."
      fi
      SLOWDLGO="YES"        
    fi
  fi

  if [ "$SLOWDLGO" = "YES" ]; then 
    sleep $DELAYSLOWDL
    USERSTOBAN=""
    LASTBAN=""
    GOTBAN=""

    for users in $TOCHECK; do
      for user in `$WHOBIN | awk -f ' ' | grep -w Dn: | grep -w $users | egrep -v $EXCLUDE3 | sort` ;do
        USERNOW="$( echo $user | awk -F"^" '{print $1}')"
        USERPID="$( echo $user | awk -F"^" '{print $2}')"
        USERSPEED="$( echo $user | awk -F"^" '{print $4}')"
        USERSPEED2="$( echo $USERSPEED | awk -F"." '{print $1}')"
        if [ "$USERSPEED2" -lt "$SPEEDLIMIT" ]; then
          if [ "$BOTSLOW" = "TRUE" ]; then
            echo "$(date +'%a %b %d %T %Y') $BOTTRIGGER \002-(SLOWDL)- $USERNOW\002 kicked for slow downloading. Only $USERSPEED k/sec" >> $GLLOG
          fi
          if [ "$DEBUG" = "TRUE" ]; then
            echo "$USERNOW is still too slow at only $USERSPEED k/sec. Killing $USERPID"
          fi
          if [ "$LOG" != "" ]; then
             echo "$(date +%x' '%T): SLOWDL: $USERNOW kicked for slowdl at $USERSPEED k/sec" >> $LOG
          fi
          if [ "$BAN" = "TRUE" ]; then
            if [ "$LASTBAN" != "$USERNOW" ]; then
              if [ "$DEBUG" = "TRUE" ]; then
                echo "Banning $USERNOW for $BANTIME seconds."
              fi
              if [ "$LOG" != "" ]; then
                echo "$(date +%x' '%T): SLOWDL: $USERNOW tempbanned for $BANTIME seconds." >> $LOG
              fi
              if [ "$TEST" != "TRUE" ]; then
              FLAGS="$(cat $USERFOLDER/$USERNOW | grep -w FLAGS | awk -F" " '{print $2}')"
              CHECK="$( echo $FLAGS | grep 6 )"
                if [ "$CHECK" != "" ]; then
                  CHECK=""
                  if [ "$DEBUG" = "TRUE" ]; then
                    echo "$USERNOW - Already deleted. Will not delete again."
                    GOTBAN="$USERNOW $GOTBAN"
                  fi
                else
                  if [ -e $USERFOLDER/$USERNOW ]; then
                    if [ "$DEBUG" = "TRUE" ]; then
                      echo "Found user in $USERFOLDER/$USERNOW. Copying to $TEMPPATH"
                    fi
                    if [ "$TEST" != "TRUE" ]; then
                      cp -f $USERFOLDER/$USERNOW $TEMPPATH/$USERNOW
                      FLAGS="$(cat $USERFOLDER/$USERNOW | grep -w FLAGS | awk -F" " '{print $2}')"
                      sed -e "s/^FLAGS $FLAGS.*/FLAGS "$FLAGS"6/" $USERFOLDER/$USERNOW > $TEMPPATH/$USERNOW.new
                      mv -f $TEMPPATH/$USERNOW.new $USERFOLDER/$USERNOW
                      GOTBAN="$USERNOW $GOTBAN"
                      echo "You were downloading too slowly. Only $USERSPEED k/sec . $BANTIME seconds tempban." > $BYEFOLDER/$USERNOW.bye
                    fi
                  fi
                fi
              fi
              LASTBAN=$USERNOW
            fi
          fi
          PIDS="$USERPID"

          if [ "$BAN" = "TRUE" ]; then
            if [ "$DEBUG" = "TRUE" ]; then
              echo "Ban mode for Slow DL is TRUE. Will check all pids of this user."
            fi
            for pids in `$WHOBIN | tr -d ' ' | grep -w $users | egrep -v $EXCLUDE1 | sort ` ;do
              USERPID="$( echo $pids | awk -F"^" '{print $2}')"
              PIDS2="$USERPID $PIDS2"
            done
            if [ "$PIDS2" = "$PIDS " ]; then
              if [ "$DEBUG" = "TRUE" ]; then
                echo "Did not find that user doing anything else on site (but being slow)."
              fi
            else
              if [ "$DEBUG" = "TRUE" ]; then
                echo "Found the user doing some other stuff. Those will be kicked too."
              fi
              PIDS=$PIDS2
              PIDS2=""
            fi
          fi

          if [ "$PIDS" != "" ]; then
            if [ "$DEBUG" = "TRUE" ]; then
              echo "Killing $PIDS now."
            fi
            if [ "$TEST" != "TRUE" ]; then
              for pid in $PIDS; do
                kill $pid
              done
            fi
          else
            if [ "$DEBUG" = "TRUE" ]; then
              echo "Cant find his pid anymore. Someone kicked him (me earlier perhaps)."
              echo "Nothing to worry about."
            fi
            if [ "$LOG" != "" ]; then
              echo "$(date +%x' '%T): SLOWDL: Cant find users pid anymore. Perhaps he logged (chicken)." >> $LOG
            fi
          fi
        fi
      done
    done

    if [ "$GOTBAN" != "" ]; then
      if [ "$DEBUG" = "TRUE" ]; then
        echo "Waiting $BANTIME to readd banned slow download users."
      fi
      sleep $BANTIME
      for user in $GOTBAN; do
        if [ -e $TEMPPATH/$user ]; then
          if [ -e $BYEFOLDER/$user.bye ]; then
            if [ "$TEST" != "TRUE" ]; then
              rm -f $BYEFOLDER/$user.bye
            fi
          fi
          mv -f $TEMPPATH/$user $USERFOLDER/$user
          if [ "$DEBUG" = "TRUE" ]; then
            echo "Restoring $user to $USERFOLDER/$user"
          fi
          if [ "$LOG" != "" ]; then
            echo "$(date +%x' '%T): SLOWDL: Restoring tempban from $user." >> $LOG
          fi
        else
          if [ "$DEBUG" = "TRUE" ]; then
            echo "Could not find $TEMPPATH/$user to restore. Guess I didnt delete him."
          fi
        fi
      done
    fi
  fi
fi

rm -f $LOCKFILE
